--[[----------------------------------------------------------------------------------------------------------------------
 File         : xRayScreenCalc.lua
 Description  : Экранный калькулятор для xRay 1.0006
 Version      : 1.0
 Last update  : 23.6.2017 
 Autors       : 7.9, Charsi (http://www.amk-team.ru/forum)
 Requirements : Notepad++ (https://notepad-plus-plus.org), LuaScript plugin for N++ (https://dail8859.github.io/LuaScript/)
--]]-----------------------------------------------------------------------------------------------------------------------

-- параметры экранов и вид адаптации
ORX = 1024	-- Original resolution X
ORY = 768	-- Original resolution Y
NRX = 1440	-- New resolution X
NRY = 900	-- New resolution Y
Mode = "XY" -- "XY" or "Xy" or "Yx"

-- напоминалка
function print_help()
    print("Параметры xRayScreenCalc:")
    print("ORX = ", ORX)
    print("ORY = ", ORY)
    print("NRX = ", NRX)
    print("NRY = ", NRY)
    print("Mode = ", Mode)
    print("")
end

-- функция расчета, TypeValue = "Cx","Lx","Cy","Ly" (C=coordinate, L=lenght)
function calc_data( TypeValue )
    if npp.BufferLangType[npp.CurrentBufferID] ~= L_XML then print("NOT MXL file!") return false end -- не XML файл
    
    local txt = editor:GetSelText()
    local Value = tonumber(txt)
    if not Value then print("selected is not number") return false end -- выделено не число
    
    local res = nil
    
    -- пробуем преобразовать    
    local ChangeX = NRX/ORX
    local ChangeY = NRY/ORY

    if Mode == "XY" then
        if TypeValue == "Cx" then res = (ORX/2-1)-((ORX/2-1)-Value)/ChangeX end
        if TypeValue == "Lx" then res = Value/ChangeX end
        if TypeValue == "Cy" then res = (ORY/2-1)-((ORY/2-1)-Value)/ChangeY end
        if TypeValue == "Ly" then res = Value/ChangeY end
        
    elseif Mode == "Xy" then
        if TypeValue == "Cx" then res = (ORX/2-1)-((ORX/2-1)-Value)*(ChangeY/ChangeX) end
        if TypeValue == "Lx" then res = Value*ChangeY/ChangeX end
        
    elseif Mode == "Yx" then
        if TypeValue == "Cy" then res = (ORY/2-1)-((ORY/2-1)-Value)*(ChangeX/ChangeY) end
        if TypeValue == "Ly" then res = Value*ChangeX/ChangeY end
    end
    
    -- в случае неудачи оставляем как есть.
    if not res then print( string.format("mismatch Mode (%s) and TypeValue (%s)", tostring(Mode), tostring(TypeValue)) ) return false end

    -- превращаем в строку с откидыванием дробной части
    editor:ReplaceSel( string.format("%.f", res) )
    return true
end

-- подписка на горячие клавиши
npp.AddShortcut("xRayScreenCalc Cx", "Alt+1", function() return calc_data("Cx") end)
npp.AddShortcut("xRayScreenCalc Lx", "Alt+2", function() return calc_data("Lx") end)
npp.AddShortcut("xRayScreenCalc Cy", "Alt+3", function() return calc_data("Cy") end)
npp.AddShortcut("xRayScreenCalc Ly", "Alt+4", function() return calc_data("Ly") end)
npp.AddShortcut("xRayScreenCalc параметры", "Alt+5", function() print_help() return false end)



---------------------------------------------------------------------------------------------------------------
--        УСТАНОВКА
-- 1. Установить "Notepad++".
-- 2. Добавить в "Notepad++" плагин "LuaScript plugin for N++" (http://dail8859.github.io/LuaScript).
-- 3. Запустить "Notepad++" с этим плагином.
-- 4. Скопировать "этот файл" в папку "C:\Users\..ваш_пользователь..\AppData\Roaming\Notepad++\plugins\config\"
-- 5. В этой-же папке, в файл "startup.lua", добавить строку: 
-- dofile([[C:\Users\..ваш_пользователь..\AppData\Roaming\Notepad++\plugins\config\xRayScreenCalc.lua]])
-- 6. Разбиндить в "Опции/Горячии клавиши/Main menu" сочетания клавиш Alt+1, Alt+2, Alt+3, Alt+4, Alt+5
-- 7. Перезапустить "Notepad++". В меню "Плагины/LuaScript" появится пять дополнительных пунктов с "горячими" клавишами. Рабочие сообщения скрипта будут выводиться в консоль, там-же можно присвоить переменным скрипта другие значения.

--[[      НАСТРОЙКА.
! Все параметры устанавливаются в переменных под текстом "параметры экранов и вид адаптации" в начале файла.
1. Выясняем параметры экрана оригинального оформления.
2. Выясняем параметры нового экрана.
3. Присваиваем значения разрешениий "Оригинального" и "Нового" экранов соответствующим переменным.
4. Решаем какой вид адаптации будет применён:
	XY -- с сохранением пиксельных размеров оригинала;
	Yx -- с подстройкой изменения высоты под изменения ширины;
	Xy -- с подстройкой изменения ширины под изменение высоты.
5. Присваиваем переменной Mode нужное значение ("XY","Xy","Yx")

          ИСПОЛЬЗОВАНИЕ.
1. 2хЛКМ выделяем нужное значение-число.
2. Определяем смысл этого числа: координата (x, y) или длина (width, height).
4. В соответствии со "смыслом этого числа" используем соответствующую комбинацию клавиш и заменяем текущее значение:
	Alt+1 если выделенное число координата по X	-- Cx
	Alt+2 если выделенное число длина по X		-- Lx
	Alt+3 если выделенное число координата по Y	-- Cy
	Alt+4 если выделенное число длина по Y		-- Ly
5. И так со всеми:
	-- "x", "y", "width" и "height"	если вид адаптации "XY"
	-- "x" и "width"				если вид адаптации "Xy"
	-- "y" и "height"				если вид адаптации "Yx"

	
=========================
Алгоритм расчёта - 7.9
Программирование - Charsi




ДОПОЛНИТЕЛЬНО

1. Адаптация без изменения текстур возможна благодаря наличию у движка "X-Ray" способности "тянуть" текстуры. При адаптациях, везде где описывается вывод текстуры, в теге элемента изображения должны присутствовать параметры: x="" y="" width="" height="" и stretch="1".

2. Если отношения сторон нового экрана 16х10 или 16х9 - имя нового "файла настроек" должно иметь окончание "_16".

3. Если в исходном файле у параметра не указаны значения "width" и "height" - значит "stretch" не применяется. С учётом пункта 1 -- их надо добавить. В этом случае значения для параметров "width" и "height" берутся из свойств текстуры.

4. Иногда значения x="" и y="" имеют смысл длин, а не координат - это значит, что данный элемент выводится относительно положения другого элемента - понятно, что результат, в этом случае, надо считать для длин.

5. Иногда, может встретится значение которое нет смысла или не нужно адаптировать -- сообразите.

6. Иногда бывают значения которые почему-то не изменяются - их всё равно можно подстроить: или в параметрах текстуры, или можно попробовать заменить этот элемент на "auto_static".

7. Даже для "соседних" ("5х4 и 4х3" или "16х10 и 16х9") пропорций имеет смысл делать отдельную адаптацию - разница в изображении слишком большая. Разница между экранами "соседних" разрешений примерно 10% -- для глаза это много.

]]